% [f, G, Gamma, Setup, ModalParams, PlotParams] = txf(filename)
%
% Reads binary data from TXF data files
%   Input: 
%     filename - TXF filename and path (matlab text string) 
%
%   Outputs: 
%     f           - Frequency vector in Hz
%     G           - Frequency response vector in m/N (complex) 
%     Gamma       - Coherence function (real)
%     Setup       - Mesurement setup parameters
%     ModalParams - Modal parameter estimation
%     PlotParams  - Plotting Parameters
%
% If called without any return parameters the measurement signals will be plotted using 
% the limits that were applied when the measurement was saved.

function [f, G, Gamma, Setup, ModalParams, PlotParams]=txf(filename)

% add default extension of 'TXF' if no extension is specified
pp=findstr(filename,'.');			
if (isempty(pp))
 filename=[deblank(filename),'.txf'];
end

fid=fopen(deblank(filename),'rb','ieee-le');

strsize=26;
descrsize = 101;
maxpath = 260;
gain=[0.5,1,2,5,10,20,50,100,1];
MeasType={'Tool Direct - X Direction', 'Tool Direct - Y Direction', 'Workpiece Direct - X Direction', 'Workpiece Direct - Y Direction', 'Tool-Workpiece Cross - X Direction', 'Tool-Workpiece Cross - Y Direction'};

Setup.Header      = char([fread(fid,strsize,'char')]');

if (strncmp(Setup.Header, 'TXF DATA FILE', 13)) % Version 100000 and greater
 TxfVerNo = fread(fid,1,'int32');
 bandwidth=[20e3,10e3,5e3,2e3,1e3,500,200,100];
 
 Setup.Description = char([fread(fid,descrsize,'char')]');
 Setup.Tool        = char([fread(fid,descrsize,'char')]');
 Setup.Machine     = char([fread(fid,descrsize,'char')]');
 Setup.UnitDisp    = fread(fid,1,'int32');
 
 for i=1:3
  Setup.SensorHamName(i,:) = char([fread(fid,descrsize,'char')]');
  Setup.SensorAccName(i,:) = char([fread(fid,descrsize,'char')]');
  Setup.Cal(i,:)        = [fread(fid,2,'float32')]';
  Setup.UnitNum(i,:)    = [fread(fid,2,'int32')]';
  Setup.UnitDen(i,:)    = [fread(fid,2,'int32')]';
  index                 = [fread(fid,2,'int32')]';
  Setup.Gain(i,:)       = gain(index+1);
  clear index;
  Setup.Trigger(i)      = fread(fid,1,'float32');
  Setup.Window(i,:)     = [fread(fid,2,'int32')]';
  Setup.Over(i,:)       = [fread(fid,2,'int32')];
  Setup.Under(i,:)      = [fread(fid,2,'int32')];
  Setup.Agnd(i,:)       = [fread(fid,2,'int32')];
  Setup.Icp(i,:)        = [fread(fid,2,'int32')];
  Setup.HighPass(i,:)   = [fread(fid,2,'int32')];
  Setup.Average(i)   	= fread(fid,1,'int32');
  Setup.Integration(i)  = fread(fid,1,'int32');
  Setup.AutoscaleLims.min(i)      = fread(fid,1,'float32');
  Setup.AutoscaleLims.max(i)      = fread(fid,1,'float32');
  Setup.AutoscaleFixed.xmin(i)    = fread(fid,1,'float32');
  Setup.AutoscaleFixed.xmax(i)    = fread(fid,1,'float32');
  Setup.AutoscaleFixed.xdiv(i)    = fread(fid,1,'int32');
  Setup.AutoscaleFixed.ymin(i,1)  = fread(fid,1,'float32');
  Setup.AutoscaleFixed.ymin(i,2)  = fread(fid,1,'float32');
  Setup.AutoscaleFixed.ymax(i,1)  = fread(fid,1,'float32');
  Setup.AutoscaleFixed.ymax(i,2)  = fread(fid,1,'float32');
  Setup.AutoscaleFixed.ydiv(i,1)  = fread(fid,1,'int32');
  Setup.AutoscaleFixed.ydiv(i,2)  = fread(fid,1,'int32');
  Setup.AutoscaleMethod(i)        = fread(fid,1,'int32');
  
  Setup.PreTrigTime(i)  = fread(fid,1,'float32');
  Setup.ImpactStart(i)  = fread(fid,1,'float32');
  Setup.ImpactDur(i)    = fread(fid,1,'float32');
  Setup.ExpStart(i)     = fread(fid,1,'float32');
  Setup.ExpTau(i)       = fread(fid,1,'float32');
  Setup.SignalAcq(i)    = fread(fid,1,'float32');
  Setup.UnderVolt(i)    = fread(fid,1,'int32');
 end

 Setup.TrigSource  = fread(fid,1,'int32');
 index             = fread(fid,1,'int32');
 Setup.Bandwidth   = bandwidth(index+1);
 clear index;
 Setup.FftSize     = 2^fread(fid,1,'int32');
 
 for i=1:6
  Setup.MeasEnable(i)  = fread(fid,1,'int32');
  Setup.MeasStimDir(i) = fread(fid,1,'int32');
  Setup.MeasRespDir(i) = fread(fid,1,'int32');
  Setup.MeasStimLoc(i) = fread(fid,1,'int32');
 end
 Setup.MeasActive = fread(fid,1,'int32');
     
 Cutdata.Diameter             = fread(fid,1,'float32');
 Cutdata.UnitDiameter         = fread(fid,1,'int32');
 Cutdata.Teeth                = fread(fid,1,'int32');
 Cutdata.Chipload             = fread(fid,1,'float32');
 Cutdata.UnitChipload         = fread(fid,1,'int32');
 Cutdata.Immersion            = fread(fid,1,'float32');
 Cutdata.UnitImmersion        = fread(fid,1,'int32');
 Cutdata.Millmode             = fread(fid,1,'int32');
 Cutdata.Milldir              = fread(fid,1,'float32');
 Cutdata.SweepDir             = fread(fid,1,'int32');   % new in version 100000
 Cutdata.SweepDirSteps        = fread(fid,1,'float32'); % new in version 100000
 Cutdata.SweepImmersion       = fread(fid,1,'int32');   % new in Version 100000
 Cutdata.SweepImmersionSteps  = fread(fid,1,'float32'); % new in Version 100000

 Stability.RelFrf             = fread(fid,6,'int32');
 for i=1:3
  Stability.FreqRange(i,1)    = fread(fid,1,'float32');  
  Stability.FreqRange(i,2)    = fread(fid,1,'float32');  
 end   
 Stability.Symmetric          = fread(fid,1,'int32');
 Stability.Speed              = fread(fid,1,'float32');

 if (TxfVerNo>=100001)
   Stability.TorqueEn         = fread(fid,1,'int32');           % new in version 100001
   Stability.MachineName      = char([fread(fid,256,'char')]'); % new in version 100001
   for i=1:10                                                   % new in version 100001
     Stability.TorqueLimit(i)   = fread(fid,1,'float32');       % new in version 100001
     Stability.TorqueSpeed(i)   = fread(fid,1,'int32');         % new in version 100001
   end                                                          % new in version 100001
 else
   Stability.Power              = fread(fid,1,'float32');
   Stability.UnitPower          = fread(fid,1,'int32');
   Stability.PowerEn            = fread(fid,1,'int32');
   Stability.Torque             = fread(fid,1,'float32');
   Stability.UnitTorque         = fread(fid,1,'int32');
   Stability.TorqueEn           = fread(fid,1,'int32'); 
   Stability.BaseSpeed          = fread(fid,1,'float32');
   Stability.BaseSpeedEn        = fread(fid,1,'int32'); 
 end

 Stability.SurfaceSpeed       = fread(fid,1,'float32');
 Stability.UnitSurfaceSpeed   = fread(fid,1,'int32');
 Stability.SurfaceSpeedEn     = fread(fid,1,'int32');
 Stability.FluteLen           = fread(fid,1,'float32');
 Stability.UnitFluteLen       = fread(fid,1,'int32');
 Stability.FluteLenEn         = fread(fid,1,'int32');
 Stability.SpindleSpeedLimit  = fread(fid,1,'float32');
 Stability.SpindleInterface   = fread(fid,1,'int32');
 Stability.GageLength         = fread(fid,1,'float32');
 Stability.UnitGageLen        = fread(fid,1,'int32');
 Stability.UncertainSpeed     = fread(fid,1,'float32'); % new in version 100000
 Stability.UncertainDepth     = fread(fid,1,'float32'); % new in version 100000
 Stability.UncertainResonance = fread(fid,1,'float32'); % new in version 100000

 Stability.Material.Name      = char([fread(fid,descrsize,'char')]');
 Stability.Material.Ks        = fread(fid,1,'float64');
 Stability.Material.PDcoeff   = fread(fid,1,'float64');
 Stability.Material.PDlambda  = fread(fid,1,'float64');
 Stability.Material.IncludePD = fread(fid,1,'int32');
 fread(fid,2,'float64');
 
 for i=1:6
  PlotParams.ReIm(i).Autoscale = fread(fid,1,'int32');
  PlotParams.ReIm(i).Xmin   = fread(fid,1,'float32');
  PlotParams.ReIm(i).Xmax   = fread(fid,1,'float32');
  PlotParams.ReIm(i).Xdiv   = fread(fid,1,'int32');
  PlotParams.ReIm(i).Ymin(1)= fread(fid,1,'float32');
  PlotParams.ReIm(i).Ymin(2)= fread(fid,1,'float32');
  PlotParams.ReIm(i).Ymax(1)= fread(fid,1,'float32');
  PlotParams.ReIm(i).Ymax(2)= fread(fid,1,'float32');
  PlotParams.ReIm(i).Ydiv(1)= fread(fid,1,'int32');
  PlotParams.ReIm(i).Ydiv(2)= fread(fid,1,'int32');
 end 
  
 for i=1:6
  PlotParams.CoMag(i).Autoscale = fread(fid,1,'int32');
  PlotParams.CoMag(i).Xmin   = fread(fid,1,'float32');
  PlotParams.CoMag(i).Xmax   = fread(fid,1,'float32');
  PlotParams.CoMag(i).Xdiv   = fread(fid,1,'int32');
  PlotParams.CoMag(i).Ymin(1)= fread(fid,1,'float32');
  PlotParams.CoMag(i).Ymin(2)= fread(fid,1,'float32');
  PlotParams.CoMag(i).Ymax(1)= fread(fid,1,'float32');
  PlotParams.CoMag(i).Ymax(2)= fread(fid,1,'float32');
  PlotParams.CoMag(i).Ydiv(1)= fread(fid,1,'int32');
  PlotParams.CoMag(i).Ydiv(2)= fread(fid,1,'int32');
 end 
 
 for i=1:2
  PlotParams.RelFrf(i).Autoscale = fread(fid,1,'int32');
  PlotParams.RelFrf(i).Xmin   = fread(fid,1,'float32');
  PlotParams.RelFrf(i).Xmax   = fread(fid,1,'float32');
  PlotParams.RelFrf(i).Xdiv   = fread(fid,1,'int32');
  PlotParams.RelFrf(i).Ymin(1)= fread(fid,1,'float32');
  PlotParams.RelFrf(i).Ymin(2)= fread(fid,1,'float32');
  PlotParams.RelFrf(i).Ymax(1)= fread(fid,1,'float32');
  PlotParams.RelFrf(i).Ymax(2)= fread(fid,1,'float32');
  PlotParams.RelFrf(i).Ydiv(1)= fread(fid,1,'int32');
  PlotParams.RelFrf(i).Ydiv(2)= fread(fid,1,'int32');
 end 
 
 for i=1:3
  PlotParams.Stability(i).Autoscale = fread(fid,1,'int32');
  PlotParams.Stability(i).Xmin   = fread(fid,1,'float32');
  PlotParams.Stability(i).Xmax   = fread(fid,1,'float32');
  PlotParams.Stability(i).Xdiv   = fread(fid,1,'int32');
  PlotParams.Stability(i).Ymin(1)= fread(fid,1,'float32');
  PlotParams.Stability(i).Ymin(2)= fread(fid,1,'float32');
  PlotParams.Stability(i).Ymax(1)= fread(fid,1,'float32');
  PlotParams.Stability(i).Ymax(2)= fread(fid,1,'float32');
  PlotParams.Stability(i).Ydiv(1)= fread(fid,1,'int32');
  PlotParams.Stability(i).Ydiv(2)= fread(fid,1,'int32');
 end 
 
 
 Time.Year        = fread(fid,1,'int16');
 Time.Month       = fread(fid,1,'int16');
 Time.Dayofweek   = fread(fid,1,'int16');
 Time.Day         = fread(fid,1,'int16');
 Time.Hour        = fread(fid,1,'int16');
 Time.minute      = fread(fid,1,'int16');
 Time.Second      = fread(fid,1,'int16');
 Time.Millisecond = fread(fid,1,'int16');
 
 for i=1:2
  ModalParams(i).ModeCount        = fread(fid,1,'int32');
  if (ModalParams(i).ModeCount > 0)
   for (imode=1:ModalParams(i).ModeCount)
    ModalParams(i).wn(imode,1)          = fread(fid,1,'float32');
    ModalParams(i).k(imode,1)           = fread(fid,1,'float32');
    ModalParams(i).zeta(imode,1)        = fread(fid,1,'float32');
    ModalParams(i).wmin(imode,1)        = fread(fid,1,'float32');
    ModalParams(i).wmax(imode,1)        = fread(fid,1,'float32');
   end
  end
 end

 df = fread(fid,1,'float64');
 imeas=find(Setup.MeasEnable ~= 0);
 G=zeros(2*13109,6);
 Gamma=zeros(13109,6);
 for i=1:length(imeas)
  G(:,imeas(i)) = fread(fid,2*13109,'float32');
  Gamma(:,imeas(i)) = fread(fid, 13109, 'float32');
 end 

elseif (strncmp(Setup.Header, 'TXF VER 6.3', 11))
 TxfVerNo = 6;
 bandwidth=[20e3,10e3,5e3,2e3,1e3,500,200,100];
 
 Setup.Description = char([fread(fid,descrsize,'char')]');
 Setup.Tool        = char([fread(fid,descrsize,'char')]');
 Setup.Machine     = char([fread(fid,descrsize,'char')]');
 Setup.UnitDisp    = fread(fid,1,'int32');
 
 for i=1:3
  Setup.SensorHamName(i,:) = char([fread(fid,descrsize,'char')]'); % new in Version 6.X
  Setup.SensorAccName(i,:) = char([fread(fid,descrsize,'char')]'); % new in Version 6.X
  Setup.Cal(i,:)        = [fread(fid,2,'float32')]';
  Setup.UnitNum(i,:)    = [fread(fid,2,'int32')]';
  Setup.UnitDen(i,:)    = [fread(fid,2,'int32')]';
  index                 = [fread(fid,2,'int32')]';
  Setup.Gain(i,:)       = gain(index+1);
  clear index;
  Setup.Trigger(i)      = fread(fid,1,'float32');
  Setup.Window(i,:)     = [fread(fid,2,'int32')]';
  Setup.Over(i,:)       = [fread(fid,2,'int32')];
  Setup.Under(i,:)      = [fread(fid,2,'int32')];
  Setup.Agnd(i,:)       = [fread(fid,2,'int32')];
  Setup.Icp(i,:)        = [fread(fid,2,'int32')];
  Setup.HighPass(i,:)   = [fread(fid,2,'int32')];
  Setup.Average(i)   	= fread(fid,1,'int32');
  Setup.Integration(i)  = fread(fid,1,'int32');
  Setup.AutoscaleLims.min(i)      = fread(fid,1,'float32');
  Setup.AutoscaleLims.max(i)      = fread(fid,1,'float32');
  Setup.AutoscaleFixed.xmin(i)    = fread(fid,1,'float32');
  Setup.AutoscaleFixed.xmax(i)    = fread(fid,1,'float32');
  Setup.AutoscaleFixed.xdiv(i)    = fread(fid,1,'int32');
  Setup.AutoscaleFixed.ymin(i,1)  = fread(fid,1,'float32');
  Setup.AutoscaleFixed.ymin(i,2)  = fread(fid,1,'float32');
  Setup.AutoscaleFixed.ymax(i,1)  = fread(fid,1,'float32');
  Setup.AutoscaleFixed.ymax(i,2)  = fread(fid,1,'float32');
  Setup.AutoscaleFixed.ydiv(i,1)  = fread(fid,1,'int32');
  Setup.AutoscaleFixed.ydiv(i,2)  = fread(fid,1,'int32');
  Setup.AutoscaleMethod(i)        = fread(fid,1,'int32');
  
  Setup.PreTrigTime(i)  = fread(fid,1,'float32');
  Setup.ImpactStart(i)  = fread(fid,1,'float32');
  Setup.ImpactDur(i)    = fread(fid,1,'float32');
  Setup.ExpStart(i)     = fread(fid,1,'float32');
  Setup.ExpTau(i)       = fread(fid,1,'float32');
  Setup.SignalAcq(i)    = fread(fid,1,'float32');
  Setup.UnderVolt(i)    = fread(fid,1,'int32');
 end

 Setup.TrigSource  = fread(fid,1,'int32');
 index             = fread(fid,1,'int32');
 Setup.Bandwidth   = bandwidth(index+1);
 clear index;
 Setup.FftSize     = 2^fread(fid,1,'int32');
 
 for i=1:6
  Setup.MeasEnable(i)  = fread(fid,1,'int32');
  Setup.MeasStimDir(i) = fread(fid,1,'int32');
  Setup.MeasRespDir(i) = fread(fid,1,'int32');
  Setup.MeasStimLoc(i) = fread(fid,1,'int32');
 end
 Setup.MeasActive = fread(fid,1,'int32');
     
 Cutdata.Diameter             = fread(fid,1,'float32');
 Cutdata.UnitDiameter         = fread(fid,1,'int32');
 Cutdata.Teeth                = fread(fid,1,'int32');
 Cutdata.Chipload             = fread(fid,1,'float32');
 Cutdata.UnitChipload         = fread(fid,1,'int32');
 Cutdata.Immersion            = fread(fid,1,'float32');
 Cutdata.UnitImmersion        = fread(fid,1,'int32');
 Cutdata.Millmode             = fread(fid,1,'int32');
 Cutdata.Milldir              = fread(fid,1,'float32');

 Stability.RelFrf             = fread(fid,6,'int32');
 for i=1:3
  Stability.FreqRange(i,1)    = fread(fid,1,'float32');  
  Stability.FreqRange(i,2)    = fread(fid,1,'float32');  
 end   
 Stability.Symmetric          = fread(fid,1,'int32');
 Stability.Speed              = fread(fid,1,'float32');
 Stability.Power              = fread(fid,1,'float32');
 Stability.UnitPower          = fread(fid,1,'int32');
 Stability.PowerEn            = fread(fid,1,'int32');
 Stability.Torque             = fread(fid,1,'float32');
 Stability.UnitTorque         = fread(fid,1,'int32');
 Stability.TorqueEn           = fread(fid,1,'int32'); 
 Stability.BaseSpeed          = fread(fid,1,'float32');
 Stability.BaseSpeedEn        = fread(fid,1,'int32'); 
 Stability.SurfaceSpeed       = fread(fid,1,'float32');
 Stability.UnitSurfaceSpeed   = fread(fid,1,'int32');
 Stability.SurfaceSpeedEn     = fread(fid,1,'int32');
 Stability.FluteLen           = fread(fid,1,'float32'); % new in Version 6.X
 Stability.UnitFluteLen       = fread(fid,1,'int32');   % new in Version 6.X
 Stability.FluteLenEn         = fread(fid,1,'int32');   % new in Version 6.X
 Stability.SpindleSpeedLimit  = fread(fid,1,'float32'); % new in Version 6.X
 Stability.SpindleInterface   = fread(fid,1,'int32');   % new in Version 6.X
 Stability.GageLength         = fread(fid,1,'float32'); % new in Version 6.X
 Stability.UnitGageLen        = fread(fid,1,'int32');   % new in Version 6.X

 Stability.Material.Name      = char([fread(fid,descrsize,'char')]');
 Stability.Material.Ks        = fread(fid,1,'float64');
 Stability.Material.PDcoeff   = fread(fid,1,'float64');
 Stability.Material.PDlambda  = fread(fid,1,'float64');
 Stability.Material.IncludePD = fread(fid,1,'int32');
 fread(fid,2,'float64');
 
 for i=1:6
  PlotParams.ReIm(i).Autoscale = fread(fid,1,'int32');
  PlotParams.ReIm(i).Xmin   = fread(fid,1,'float32');
  PlotParams.ReIm(i).Xmax   = fread(fid,1,'float32');
  PlotParams.ReIm(i).Xdiv   = fread(fid,1,'int32');
  PlotParams.ReIm(i).Ymin(1)= fread(fid,1,'float32');
  PlotParams.ReIm(i).Ymin(2)= fread(fid,1,'float32');
  PlotParams.ReIm(i).Ymax(1)= fread(fid,1,'float32');
  PlotParams.ReIm(i).Ymax(2)= fread(fid,1,'float32');
  PlotParams.ReIm(i).Ydiv(1)= fread(fid,1,'int32');
  PlotParams.ReIm(i).Ydiv(2)= fread(fid,1,'int32');
 end 
  
 for i=1:6
  PlotParams.CoMag(i).Autoscale = fread(fid,1,'int32');
  PlotParams.CoMag(i).Xmin   = fread(fid,1,'float32');
  PlotParams.CoMag(i).Xmax   = fread(fid,1,'float32');
  PlotParams.CoMag(i).Xdiv   = fread(fid,1,'int32');
  PlotParams.CoMag(i).Ymin(1)= fread(fid,1,'float32');
  PlotParams.CoMag(i).Ymin(2)= fread(fid,1,'float32');
  PlotParams.CoMag(i).Ymax(1)= fread(fid,1,'float32');
  PlotParams.CoMag(i).Ymax(2)= fread(fid,1,'float32');
  PlotParams.CoMag(i).Ydiv(1)= fread(fid,1,'int32');
  PlotParams.CoMag(i).Ydiv(2)= fread(fid,1,'int32');
 end 
 
 for i=1:2
  PlotParams.RelFrf(i).Autoscale = fread(fid,1,'int32');
  PlotParams.RelFrf(i).Xmin   = fread(fid,1,'float32');
  PlotParams.RelFrf(i).Xmax   = fread(fid,1,'float32');
  PlotParams.RelFrf(i).Xdiv   = fread(fid,1,'int32');
  PlotParams.RelFrf(i).Ymin(1)= fread(fid,1,'float32');
  PlotParams.RelFrf(i).Ymin(2)= fread(fid,1,'float32');
  PlotParams.RelFrf(i).Ymax(1)= fread(fid,1,'float32');
  PlotParams.RelFrf(i).Ymax(2)= fread(fid,1,'float32');
  PlotParams.RelFrf(i).Ydiv(1)= fread(fid,1,'int32');
  PlotParams.RelFrf(i).Ydiv(2)= fread(fid,1,'int32');
 end 
 
 for i=1:3
  PlotParams.Stability(i).Autoscale = fread(fid,1,'int32');
  PlotParams.Stability(i).Xmin   = fread(fid,1,'float32');
  PlotParams.Stability(i).Xmax   = fread(fid,1,'float32');
  PlotParams.Stability(i).Xdiv   = fread(fid,1,'int32');
  PlotParams.Stability(i).Ymin(1)= fread(fid,1,'float32');
  PlotParams.Stability(i).Ymin(2)= fread(fid,1,'float32');
  PlotParams.Stability(i).Ymax(1)= fread(fid,1,'float32');
  PlotParams.Stability(i).Ymax(2)= fread(fid,1,'float32');
  PlotParams.Stability(i).Ydiv(1)= fread(fid,1,'int32');
  PlotParams.Stability(i).Ydiv(2)= fread(fid,1,'int32');
 end 
 
 
 Time.Year        = fread(fid,1,'int16');
 Time.Month       = fread(fid,1,'int16');
 Time.Dayofweek   = fread(fid,1,'int16');
 Time.Day         = fread(fid,1,'int16');
 Time.Hour        = fread(fid,1,'int16');
 Time.minute      = fread(fid,1,'int16');
 Time.Second      = fread(fid,1,'int16');
 Time.Millisecond = fread(fid,1,'int16');
 
 for i=1:2
  ModalParams(i).ModeCount        = fread(fid,1,'int32');
  if (ModalParams(i).ModeCount > 0)
   for (imode=1:ModalParams(i).ModeCount)
    ModalParams(i).wn(imode,1)          = fread(fid,1,'float32');
    ModalParams(i).k(imode,1)           = fread(fid,1,'float32');
    ModalParams(i).zeta(imode,1)        = fread(fid,1,'float32');
    ModalParams(i).wmin(imode,1)        = fread(fid,1,'float32');
    ModalParams(i).wmax(imode,1)        = fread(fid,1,'float32');
   end
  end
 end

 df = fread(fid,1,'float64');
 imeas=find(Setup.MeasEnable ~= 0);
 G=zeros(2*13109,6);
 Gamma=zeros(13109,6);
 for i=1:length(imeas)
  G(:,imeas(i)) = fread(fid,2*13109,'float32');
  Gamma(:,imeas(i)) = fread(fid, 13109, 'float32');
 end 

elseif (strncmp(Setup.Header, 'TXF VER 5.0', 11))
 TxfVerNo = 5;
 bandwidth=[20e3,10e3,5e3,2e3,1e3,500,200,100];
 
 Setup.Description = char([fread(fid,descrsize,'char')]');
 Setup.Tool        = char([fread(fid,descrsize,'char')]');
 Setup.Machine     = char([fread(fid,descrsize,'char')]');
 Setup.UnitDisp    = fread(fid,1,'int32');
 
 for i=1:3
  Setup.Cal(i,:)        = [fread(fid,2,'float32')]';
  Setup.UnitNum(i,:)    = [fread(fid,2,'int32')]';
  Setup.UnitDen(i,:)    = [fread(fid,2,'int32')]';
  index                 = [fread(fid,2,'int32')]';
  Setup.Gain(i,:)       = gain(index+1);
  clear index;
  Setup.Trigger(i)      = fread(fid,1,'float32');
  Setup.Window(i,:)     = [fread(fid,2,'int32')]';
  Setup.Over(i,:)       = [fread(fid,2,'int32')];
  Setup.Under(i,:)      = [fread(fid,2,'int32')];
  Setup.Agnd(i,:)       = [fread(fid,2,'int32')];
  Setup.Icp(i,:)        = [fread(fid,2,'int32')];
  Setup.HighPass(i,:)   = [fread(fid,2,'int32')];
  Setup.Average(i)   	= fread(fid,1,'int32');
  Setup.Integration(i)  = fread(fid,1,'int32');
  Setup.AutoscaleLims.min(i)      = fread(fid,1,'float32');
  Setup.AutoscaleLims.max(i)      = fread(fid,1,'float32');
  Setup.AutoscaleFixed.xmin(i)    = fread(fid,1,'float32');
  Setup.AutoscaleFixed.xmax(i)    = fread(fid,1,'float32');
  Setup.AutoscaleFixed.xdiv(i)    = fread(fid,1,'int32');
  Setup.AutoscaleFixed.ymin(i,1)  = fread(fid,1,'float32');
  Setup.AutoscaleFixed.ymin(i,2)  = fread(fid,1,'float32');
  Setup.AutoscaleFixed.ymax(i,1)  = fread(fid,1,'float32');
  Setup.AutoscaleFixed.ymax(i,2)  = fread(fid,1,'float32');
  Setup.AutoscaleFixed.ydiv(i,1)  = fread(fid,1,'int32');
  Setup.AutoscaleFixed.ydiv(i,2)  = fread(fid,1,'int32');
  Setup.AutoscaleMethod(i)        = fread(fid,1,'int32');
  
  Setup.PreTrigTime(i)  = fread(fid,1,'float32');
  Setup.ImpactStart(i)  = fread(fid,1,'float32');
  Setup.ImpactDur(i)    = fread(fid,1,'float32');
  Setup.ExpStart(i)     = fread(fid,1,'float32');
  Setup.ExpTau(i)       = fread(fid,1,'float32');
  Setup.SignalAcq(i)    = fread(fid,1,'float32');
  Setup.UnderVolt(i)    = fread(fid,1,'int32');
 end

 Setup.TrigSource  = fread(fid,1,'int32');
 index             = fread(fid,1,'int32');
 Setup.Bandwidth   = bandwidth(index+1);
 clear index;
 Setup.FftSize     = 2^fread(fid,1,'int32');
 
 for i=1:6
  Setup.MeasEnable(i)  = fread(fid,1,'int32');
  Setup.MeasStimDir(i) = fread(fid,1,'int32');
  Setup.MeasRespDir(i) = fread(fid,1,'int32');
  Setup.MeasStimLoc(i) = fread(fid,1,'int32');
 end
 Setup.MeasActive = fread(fid,1,'int32');
     
 Cutdata.Diameter             = fread(fid,1,'float32');
 Cutdata.UnitDiameter         = fread(fid,1,'int32');
 Cutdata.Teeth                = fread(fid,1,'int32');
 Cutdata.Chipload             = fread(fid,1,'float32');
 Cutdata.UnitChipload         = fread(fid,1,'int32');
 Cutdata.Immersion            = fread(fid,1,'float32');
 Cutdata.UnitImmersion        = fread(fid,1,'int32');
 Cutdata.Millmode             = fread(fid,1,'int32');
 Cutdata.Milldir              = fread(fid,1,'float32');

 Stability.RelFrf             = fread(fid,6,'int32');
 for i=1:3
  Stability.FreqRange(i,1)    = fread(fid,1,'float32');  
  Stability.FreqRange(i,2)    = fread(fid,1,'float32');  
 end   
 Stability.Symmetric          = fread(fid,1,'int32');
 Stability.Speed              = fread(fid,1,'float32');
 Stability.Power              = fread(fid,1,'float32');
 Stability.UnitPower          = fread(fid,1,'int32');
 Stability.PowerEn            = fread(fid,1,'int32');
 Stability.Torque             = fread(fid,1,'float32');
 Stability.UnitTorque         = fread(fid,1,'int32');
 Stability.TorqueEn           = fread(fid,1,'int32'); 
 Stability.BaseSpeed          = fread(fid,1,'float32');
 Stability.BaseSpeedEn        = fread(fid,1,'int32'); 
 Stability.SurfaceSpeed       = fread(fid,1,'float32');
 Stability.UnitSurfaceSpeed   = fread(fid,1,'int32');
 Stability.SurfaceSpeedEn     = fread(fid,1,'int32'); 

 Stability.Material.Name      = char([fread(fid,descrsize,'char')]');
 Stability.Material.Ks        = fread(fid,1,'float64');
 Stability.Material.PDcoeff   = fread(fid,1,'float64');
 Stability.Material.PDlambda  = fread(fid,1,'float64');
 Stability.Material.IncludePD = fread(fid,1,'int32');
 fread(fid,2,'float64');
 
 for i=1:6
  PlotParams.ReIm(i).Autoscale = fread(fid,1,'int32');
  PlotParams.ReIm(i).Xmin   = fread(fid,1,'float32');
  PlotParams.ReIm(i).Xmax   = fread(fid,1,'float32');
  PlotParams.ReIm(i).Xdiv   = fread(fid,1,'int32');
  PlotParams.ReIm(i).Ymin(1)= fread(fid,1,'float32');
  PlotParams.ReIm(i).Ymin(2)= fread(fid,1,'float32');
  PlotParams.ReIm(i).Ymax(1)= fread(fid,1,'float32');
  PlotParams.ReIm(i).Ymax(2)= fread(fid,1,'float32');
  PlotParams.ReIm(i).Ydiv(1)= fread(fid,1,'int32');
  PlotParams.ReIm(i).Ydiv(2)= fread(fid,1,'int32');
 end 
  
 for i=1:6
  PlotParams.CoMag(i).Autoscale = fread(fid,1,'int32');
  PlotParams.CoMag(i).Xmin   = fread(fid,1,'float32');
  PlotParams.CoMag(i).Xmax   = fread(fid,1,'float32');
  PlotParams.CoMag(i).Xdiv   = fread(fid,1,'int32');
  PlotParams.CoMag(i).Ymin(1)= fread(fid,1,'float32');
  PlotParams.CoMag(i).Ymin(2)= fread(fid,1,'float32');
  PlotParams.CoMag(i).Ymax(1)= fread(fid,1,'float32');
  PlotParams.CoMag(i).Ymax(2)= fread(fid,1,'float32');
  PlotParams.CoMag(i).Ydiv(1)= fread(fid,1,'int32');
  PlotParams.CoMag(i).Ydiv(2)= fread(fid,1,'int32');
 end 
 
 for i=1:2
  PlotParams.RelFrf(i).Autoscale = fread(fid,1,'int32');
  PlotParams.RelFrf(i).Xmin   = fread(fid,1,'float32');
  PlotParams.RelFrf(i).Xmax   = fread(fid,1,'float32');
  PlotParams.RelFrf(i).Xdiv   = fread(fid,1,'int32');
  PlotParams.RelFrf(i).Ymin(1)= fread(fid,1,'float32');
  PlotParams.RelFrf(i).Ymin(2)= fread(fid,1,'float32');
  PlotParams.RelFrf(i).Ymax(1)= fread(fid,1,'float32');
  PlotParams.RelFrf(i).Ymax(2)= fread(fid,1,'float32');
  PlotParams.RelFrf(i).Ydiv(1)= fread(fid,1,'int32');
  PlotParams.RelFrf(i).Ydiv(2)= fread(fid,1,'int32');
 end 
 
 for i=1:3
  PlotParams.Stability(i).Autoscale = fread(fid,1,'int32');
  PlotParams.Stability(i).Xmin   = fread(fid,1,'float32');
  PlotParams.Stability(i).Xmax   = fread(fid,1,'float32');
  PlotParams.Stability(i).Xdiv   = fread(fid,1,'int32');
  PlotParams.Stability(i).Ymin(1)= fread(fid,1,'float32');
  PlotParams.Stability(i).Ymin(2)= fread(fid,1,'float32');
  PlotParams.Stability(i).Ymax(1)= fread(fid,1,'float32');
  PlotParams.Stability(i).Ymax(2)= fread(fid,1,'float32');
  PlotParams.Stability(i).Ydiv(1)= fread(fid,1,'int32');
  PlotParams.Stability(i).Ydiv(2)= fread(fid,1,'int32');
 end 
 
 
 Time.Year        = fread(fid,1,'int16');
 Time.Month       = fread(fid,1,'int16');
 Time.Dayofweek   = fread(fid,1,'int16');
 Time.Day         = fread(fid,1,'int16');
 Time.Hour        = fread(fid,1,'int16');
 Time.minute      = fread(fid,1,'int16');
 Time.Second      = fread(fid,1,'int16');
 Time.Millisecond = fread(fid,1,'int16');
 
 for i=1:2
  ModalParams(i).ModeCount        = fread(fid,1,'int32');
  if (ModalParams(i).ModeCount > 0)
   for (imode=1:ModalParams(i).ModeCount)
    ModalParams(i).wn(imode,1)          = fread(fid,1,'float32');
    ModalParams(i).k(imode,1)           = fread(fid,1,'float32');
    ModalParams(i).zeta(imode,1)        = fread(fid,1,'float32');
    ModalParams(i).wmin(imode,1)        = fread(fid,1,'float32');
    ModalParams(i).wmax(imode,1)        = fread(fid,1,'float32');
   end
  end
 end

 df = fread(fid,1,'float64');
 imeas=find(Setup.MeasEnable ~= 0);
 G=zeros(2*13109,6);
 Gamma=zeros(13109,6);
 for i=1:length(imeas)
  G(:,imeas(i)) = fread(fid,2*13109,'float32');
  Gamma(:,imeas(i)) = fread(fid, 13109, 'float32');
 end 

elseif (strncmp(Setup.Header, 'TXF VER 4.2', 11))
 TxfVerNo = 4.2;
 bandwidth=[20e3,10e3,5e3,2e3,1e3];
 
 Setup.Description = char([fread(fid,descrsize,'char')]');
 Setup.Tool        = char([fread(fid,descrsize,'char')]');
 Setup.Machine     = char([fread(fid,descrsize,'char')]');
 Setup.UnitDisp    = fread(fid,1,'int32');
 Setup.Cal         = fread(fid,2,'float32');
 Setup.UnitNum     = fread(fid,2,'int32');
 Setup.UnitDen     = fread(fid,2,'int32');
 index             = fread(fid,2,'int32');
 Setup.Gain        = gain(index+1);
 clear index;
 Setup.Trigger     = fread(fid,1,'float32');
 Setup.Window      = fread(fid,2,'int32');
 Setup.Over        = fread(fid,2,'int32');
 Setup.Under       = fread(fid,2,'int32');
 Setup.Agnd        = fread(fid,2,'int32');
 Setup.Icp         = fread(fid,2,'int32');
 Setup.HighPass    = fread(fid,2,'int32');
 Setup.TrigSource  = fread(fid,1,'int32');

 index             = fread(fid,1,'int32');
 Setup.Bandwidth   = bandwidth(index+1);
 clear index;
 Setup.FftSize     = 2^fread(fid,1,'int32');
 Setup.Average     = fread(fid,1,'int32');
 Setup.Antialias   = fread(fid,1,'int32');
 Setup.Integration = fread(fid,1,'int32');
 Setup.AutoscaleLims.min      = fread(fid,1,'float32');
 Setup.AutoscaleLims.max      = fread(fid,1,'float32');
 Setup.AutoscaleFixed.xmin    = fread(fid,1,'float32');
 Setup.AutoscaleFixed.xmax    = fread(fid,1,'float32');
 Setup.AutoscaleFixed.xdiv    = fread(fid,1,'int32');
 Setup.AutoscaleFixed.ymin(1) = fread(fid,1,'float32');
 Setup.AutoscaleFixed.ymin(2) = fread(fid,1,'float32');
 Setup.AutoscaleFixed.ymax(1) = fread(fid,1,'float32');
 Setup.AutoscaleFixed.ymax(2) = fread(fid,1,'float32');
 Setup.AutoscaleFixed.ydiv(1) = fread(fid,1,'int32');
 Setup.AutoscaleFixed.ydiv(2) = fread(fid,1,'int32');
 Setup.AutoscaleMethod        = fread(fid,1,'int32');
 
 Cutdata.Diameter             = fread(fid,1,'float32');
 Cutdata.UnitDiameter         = fread(fid,1,'int32');
 Cutdata.Teeth                = fread(fid,1,'int32');
 Cutdata.Chipload             = fread(fid,1,'float32');
 Cutdata.UnitChipload         = fread(fid,1,'int32');
 Cutdata.Immersion            = fread(fid,1,'float32');
 Cutdata.UnitImmersion        = fread(fid,1,'int32');
 Cutdata.Millmode             = fread(fid,1,'int32');
 Cutdata.Milldir              = fread(fid,1,'float32');

 Stability.DataX              = fread(fid,1,'int32');
 Stability.DataY              = fread(fid,1,'int32');
 Stability.FileX              = fread(fid,maxpath,'char');
 Stability.FileY              = fread(fid,maxpath,'char');
 Stability.FreqRange          = fread(fid,2,'float32');
 Stability.Speed              = fread(fid,1,'float32');
 Stability.Power              = fread(fid,1,'float32');
 Stability.UnitPower          = fread(fid,1,'int32');
 Stability.PowerEn            = fread(fid,1,'int32');
 Stability.Torque             = fread(fid,1,'float32');
 Stability.UnitTorque         = fread(fid,1,'int32');
 Stability.TorqueEn           = fread(fid,1,'int32'); 
 Stability.BaseSpeed          = fread(fid,1,'float32');
 Stability.BaseSpeedEn        = fread(fid,1,'int32'); 
 Stability.SurfaceSpeed       = fread(fid,1,'float32');
 Stability.UnitSurfaceSpeed   = fread(fid,1,'int32');
 Stability.SurfaceSpeedEn     = fread(fid,1,'int32'); 

 Stability.Material.Name      = char([fread(fid,descrsize,'char')]');
 Stability.Material.Ks        = fread(fid,1,'float64');
 Stability.Material.PDcoeff   = fread(fid,1,'float64');
 Stability.Material.PDlambda  = fread(fid,1,'float64');
 Stability.Material.IncludePD = fread(fid,1,'int32');
 fread(fid,2,'float64');
 
 PlotParams.ReIm.Xmin   = fread(fid,1,'float32');
 PlotParams.ReIm.Xmax   = fread(fid,1,'float32');
 PlotParams.ReIm.Xdiv   = fread(fid,1,'int32');
 PlotParams.ReIm.Ymin(1)= fread(fid,1,'float32');
 PlotParams.ReIm.Ymin(2)= fread(fid,1,'float32');
 PlotParams.ReIm.Ymax(1)= fread(fid,1,'float32');
 PlotParams.ReIm.Ymax(2)= fread(fid,1,'float32');
 PlotParams.ReIm.Ydiv(1)= fread(fid,1,'int32');
 PlotParams.ReIm.Ydiv(2)= fread(fid,1,'int32');

 PlotParams.CoMag.Xmin   = fread(fid,1,'float32');
 PlotParams.CoMag.Xmax   = fread(fid,1,'float32');
 PlotParams.CoMag.Xdiv   = fread(fid,1,'int32');
 PlotParams.CoMag.Ymin(1)= fread(fid,1,'float32');
 PlotParams.CoMag.Ymin(2)= fread(fid,1,'float32');
 PlotParams.CoMag.Ymax(1)= fread(fid,1,'float32');
 PlotParams.CoMag.Ymax(2)= fread(fid,1,'float32');
 PlotParams.CoMag.Ydiv(1)= fread(fid,1,'int32');
 PlotParams.CoMag.Ydiv(2)= fread(fid,1,'int32');

 Time.Year        = fread(fid,1,'int16');
 Time.Month       = fread(fid,1,'int16');
 Time.Dayofweek   = fread(fid,1,'int16');
 Time.Day         = fread(fid,1,'int16');
 Time.Hour        = fread(fid,1,'int16');
 Time.minute      = fread(fid,1,'int16');
 Time.Second      = fread(fid,1,'int16');
 Time.Millisecond = fread(fid,1,'int16');
 
 ModalParams.ModeCount        = fread(fid,1,'int32');
 if (ModalParams.ModeCount > 0)
  for (i=1:ModalParams.ModeCount)
   ModalParams.wn(i,1)          = fread(fid,1,'float32');
   ModalParams.k(i,1)           = fread(fid,1,'float32');
   ModalParams.zeta(i,1)        = fread(fid,1,'float32');
   ModalParams.wmin(i,1)        = fread(fid,1,'float32');
   ModalParams.wmax(i,1)        = fread(fid,1,'float32');
  end
 end

 df = fread(fid,1,'float64');
 G = fread(fid,Setup.FftSize+2,'float32');
 Gamma = fread(fid, Setup.FftSize/2+1, 'float32');
 
elseif (strncmp(Setup.Header, 'TXF VER 4.1', 11))
 TxfVerNo = 4.1;
 bandwidth=[20e3,10e3,5e3,2e3,1e3];
 
 Setup.Description = char([fread(fid,descrsize,'char')]');
 Setup.Tool        = char([fread(fid,descrsize,'char')]');
 Setup.Machine     = char([fread(fid,descrsize,'char')]');
 Setup.UnitDisp    = fread(fid,1,'int32');
 Setup.Cal         = fread(fid,2,'float32');
 Setup.UnitNum     = fread(fid,2,'int32');
 Setup.UnitDen     = fread(fid,2,'int32');
 index             = fread(fid,2,'int32');
 Setup.Gain        = gain(index+1);
 clear index;
 Setup.Trigger     = fread(fid,1,'float32');
 Setup.Window      = fread(fid,2,'int32');
 Setup.Over        = fread(fid,2,'int32');
 Setup.Under       = fread(fid,2,'int32');
 Setup.Agnd        = fread(fid,2,'int32');
 Setup.Icp         = fread(fid,2,'int32');
 Setup.HighPass    = fread(fid,2,'int32');
 Setup.TrigSource  = fread(fid,1,'int32');

 index             = fread(fid,1,'int32');
 Setup.Bandwidth   = bandwidth(index+1);
 clear index;
 Setup.FftSize     = 2^fread(fid,1,'int32');
 Setup.Average     = fread(fid,1,'int32');
 Setup.Antialias   = fread(fid,1,'int32');
 Setup.Integration = fread(fid,1,'int32');
 Setup.AutoscaleLims.min      = fread(fid,1,'float32');
 Setup.AutoscaleLims.max      = fread(fid,1,'float32');
 Setup.AutoscaleFixed.xmin    = fread(fid,1,'float32');
 Setup.AutoscaleFixed.xmax    = fread(fid,1,'float32');
 Setup.AutoscaleFixed.xdiv    = fread(fid,1,'int32');
 Setup.AutoscaleFixed.ymin(1) = fread(fid,1,'float32');
 Setup.AutoscaleFixed.ymin(2) = fread(fid,1,'float32');
 Setup.AutoscaleFixed.ymax(1) = fread(fid,1,'float32');
 Setup.AutoscaleFixed.ymax(2) = fread(fid,1,'float32');
 Setup.AutoscaleFixed.ydiv(1) = fread(fid,1,'int32');
 Setup.AutoscaleFixed.ydiv(2) = fread(fid,1,'int32');
 Setup.AutoscaleMethod        = fread(fid,1,'int32');
 
 Cutdata.Diameter             = fread(fid,1,'float32');
 Cutdata.UnitDiameter         = fread(fid,1,'int32');
 Cutdata.Teeth                = fread(fid,1,'int32');
 Cutdata.Chipload             = fread(fid,1,'float32');
 Cutdata.UnitChipload         = fread(fid,1,'int32');
 Cutdata.Immersion            = fread(fid,1,'float32');
 Cutdata.UnitImmersion        = fread(fid,1,'int32');
 Cutdata.Millmode             = fread(fid,1,'int32');
 Cutdata.Milldir              = fread(fid,1,'float32');

 Stability.DataX              = fread(fid,1,'int32');
 Stability.DataY              = fread(fid,1,'int32');
 Stability.FileX              = fread(fid,maxpath,'char');
 Stability.FileY              = fread(fid,maxpath,'char');
 Stability.FreqRange          = fread(fid,2,'float32');
 Stability.Speed              = fread(fid,1,'float32');
 Stability.Power              = fread(fid,1,'float32');
 Stability.UnitPower          = fread(fid,1,'int32');
 Stability.PowerEn            = fread(fid,1,'int32');
 Stability.Torque             = fread(fid,1,'float32');
 Stability.UnitTorque         = fread(fid,1,'int32');
 Stability.TorqueEn           = fread(fid,1,'int32'); 
 Stability.Material.Name      = char([fread(fid,descrsize,'char')]');
 Stability.Material.Ks        = fread(fid,1,'float64');
 Stability.Material.PDcoeff   = fread(fid,1,'float64');
 Stability.Material.PDlambda  = fread(fid,1,'float64');
 Stability.Material.IncludePD = fread(fid,1,'int32');
 junk                         = fread(fid,2,'float64');
 clear junk;
 
 PlotParams.ReIm.Xmin   = fread(fid,1,'float32');
 PlotParams.ReIm.Xmax   = fread(fid,1,'float32');
 PlotParams.ReIm.Xdiv   = fread(fid,1,'int32');
 PlotParams.ReIm.Ymin(1)= fread(fid,1,'float32');
 PlotParams.ReIm.Ymin(2)= fread(fid,1,'float32');
 PlotParams.ReIm.Ymax(1)= fread(fid,1,'float32');
 PlotParams.ReIm.Ymax(2)= fread(fid,1,'float32');
 PlotParams.ReIm.Ydiv(1)= fread(fid,1,'int32');
 PlotParams.ReIm.Ydiv(2)= fread(fid,1,'int32');

 PlotParams.CoMag.Xmin   = fread(fid,1,'float32');
 PlotParams.CoMag.Xmax   = fread(fid,1,'float32');
 PlotParams.CoMag.Xdiv   = fread(fid,1,'int32');
 PlotParams.CoMag.Ymin(1)= fread(fid,1,'float32');
 PlotParams.CoMag.Ymin(2)= fread(fid,1,'float32');
 PlotParams.CoMag.Ymax(1)= fread(fid,1,'float32');
 PlotParams.CoMag.Ymax(2)= fread(fid,1,'float32');
 PlotParams.CoMag.Ydiv(1)= fread(fid,1,'int32');
 PlotParams.CoMag.Ydiv(2)= fread(fid,1,'int32');

 Time.Year        = fread(fid,1,'int16');
 Time.Month       = fread(fid,1,'int16');
 Time.Dayofweek   = fread(fid,1,'int16');
 Time.Day         = fread(fid,1,'int16');
 Time.Hour        = fread(fid,1,'int16');
 Time.minute      = fread(fid,1,'int16');
 Time.Second      = fread(fid,1,'int16');
 Time.Millisecond = fread(fid,1,'int16');

 df = fread(fid,1,'float64');
 G = fread(fid,Setup.FftSize+2,'float32');
 Gamma = fread(fid, Setup.FftSize/2+1, 'float32');
 
elseif (strncmp(Setup.Header, 'TXF VER 4.0', 11))
 TxfVerNo = 4.1;
 bandwidth=[20e3,10e3,5e3,2e3,1e3];
 
 Setup.Description = char([fread(fid,descrsize,'char')]');
 Setup.Tool        = char([fread(fid,descrsize,'char')]');
 Setup.Machine     = char([fread(fid,descrsize,'char')]');
 Setup.Cal         = fread(fid,2,'float32');
 Setup.Unit(1,:)   = char([fread(fid,strsize,'char')]');
 Setup.Unit(2,:)   = char([fread(fid,strsize,'char')]);
 index             = fread(fid,2,'int32');
 Setup.Gain        = gain(index+1);
 clear index;
 Setup.Trigger     = fread(fid,1,'float32');
 Setup.Window      = fread(fid,2,'int32');
 Setup.Over        = fread(fid,2,'int32');
 Setup.Under       = fread(fid,2,'int32');
 Setup.Agnd        = fread(fid,2,'int32');
 Setup.Icp         = fread(fid,2,'int32');
 Setup.HighPass    = fread(fid,2,'int32');
 Setup.TrigSource  = fread(fid,1,'int32');

 index             = fread(fid,1,'int32');
 Setup.Bandwidth   = bandwidth(index+1);
 clear index;
 Setup.FftSize     = 2^fread(fid,1,'int32');
 Setup.Average     = fread(fid,1,'int32');
 Setup.Antialias   = fread(fid,1,'int32');
 Setup.Integration = fread(fid,1,'int32');
 Setup.AutoscaleLims.min      = fread(fid,1,'float64');
 Setup.AutoscaleLims.max      = fread(fid,1,'float64');
 Setup.AutoscaleFixed.xmin    = fread(fid,1,'float64');
 Setup.AutoscaleFixed.xmax    = fread(fid,1,'float64');
 Setup.AutoscaleFixed.xdiv    = fread(fid,1,'int32');
 Setup.AutoscaleFixed.ymin(1) = fread(fid,1,'float64');
 Setup.AutoscaleFixed.ymin(2) = fread(fid,1,'float64');
 Setup.AutoscaleFixed.ymax(1) = fread(fid,1,'float64');
 Setup.AutoscaleFixed.ymax(2) = fread(fid,1,'float64');
 Setup.AutoscaleFixed.ydiv(1) = fread(fid,1,'int32');
 Setup.AutoscaleFixed.ydiv(2) = fread(fid,1,'int32');
 Setup.AutoscaleMethod        = fread(fid,1,'int32');
 Cutdata.Diameter             = fread(fid,1,'float32');
 Cutdata.Teeth                = fread(fid,1,'int32');
 Cutdata.Chipload             = fread(fid,1,'float32');
 Cutdata.Immersion            = fread(fid,1,'float32');
 Cutdata.Millmode             = fread(fid,1,'int32');
 Cutdata.Milldir              = fread(fid,1,'float32');

 Stability.DataX              = fread(fid,1,'int32');
 Stability.DataY              = fread(fid,1,'int32');
 Stability.FileX              = fread(fid,maxpath,'char');
 Stability.FileY              = fread(fid,maxpath,'char');
 Stability.FreqRange          = fread(fid,2,'float64');
 Stability.Speed              = fread(fid,1,'float32');
 Stability.Power              = fread(fid,1,'float32');
 Stability.Torque             = fread(fid,1,'float32');
 Stability.PowerEn            = fread(fid,1,'int32');
 Stability.TorqueEn           = fread(fid,1,'int32'); 
 Stability.CutStiff           = fread(fid,1,'float32');
 Stability.MatSelect          = fread(fid,1,'int32');
 Stability.MatCount           = fread(fid,1,'int32');
 for i=1:25
  Stability.Title             = fread(fid,strsize,'char');
  Stability.CutStiff          = fread(fid,1,'float32');
 end

 PlotParams.ReIm.Xmin   = fread(fid,1,'float64');
 PlotParams.ReIm.Xmax   = fread(fid,1,'float64');
 PlotParams.ReIm.Xdiv   = fread(fid,1,'int32');
 PlotParams.ReIm.Ymin(1)= fread(fid,1,'float64');
 PlotParams.ReIm.Ymin(2)= fread(fid,1,'float64');
 PlotParams.ReIm.Ymax(1)= fread(fid,1,'float64');
 PlotParams.ReIm.Ymax(2)= fread(fid,1,'float64');
 PlotParams.ReIm.Ydiv(1)= fread(fid,1,'int32');
 PlotParams.ReIm.Ydiv(2)= fread(fid,1,'int32');

 Time.Year        = fread(fid,1,'int16');
 Time.Month       = fread(fid,1,'int16');
 Time.Dayofweek   = fread(fid,1,'int16');
 Time.Day         = fread(fid,1,'int16');
 Time.Hour        = fread(fid,1,'int16');
 Time.minute      = fread(fid,1,'int16');
 Time.Second      = fread(fid,1,'int16');
 Time.Millisecond = fread(fid,1,'int16');

 df = fread(fid,1,'float64');
 G = fread(fid,Setup.FftSize+2,'float32');
 Gamma = fread(fid, Setup.FftSize/2+1, 'float32');
 
elseif (strncmp(Setup.Header, 'TXF VER 3.0', 11) | strncmp(Setup.Header, 'TXF VER 3.1', 11))
 TxfVerNo = 3;
 bandwidth=[10e3,5e3,2.5e3,1e3];
   
 Setup.Description = char([fread(fid,descrsize,'char')]');
 Setup.Tool        = char([fread(fid,descrsize,'char')]');
 Setup.Machine     = char([fread(fid,descrsize,'char')]');
 Setup.Cal         = fread(fid,2,'float32');
 Setup.Unit(1,:)   = char([fread(fid,strsize,'char')]');
 Setup.Unit(2,:)   = char([fread(fid,strsize,'char')]);
 index             = fread(fid,2,'int32');
 Setup.Gain        = gain(index+1);
 clear index;
 Setup.Trigger     = fread(fid,1,'float32');
 Setup.Window      = fread(fid,2,'int32');
 Setup.Over        = fread(fid,2,'int32');
 Setup.Under       = fread(fid,2,'int32');
 index             = fread(fid,1,'int32');
 Setup.Bandwidth   = bandwidth(index+1);
 clear index;
 Setup.Average     = fread(fid,1,'int32');
 Setup.Antialias   = fread(fid,1,'int32');
 Setup.Integration = fread(fid,1,'int32');
 Setup.AutoscaleLims.min      = fread(fid,1,'float64');
 Setup.AutoscaleLims.max      = fread(fid,1,'float64');
 Setup.AutoscaleFixed.xmin    = fread(fid,1,'float64');
 Setup.AutoscaleFixed.xmax    = fread(fid,1,'float64');
 Setup.AutoscaleFixed.xdiv    = fread(fid,1,'int32');
 Setup.AutoscaleFixed.ymin(1) = fread(fid,1,'float64');
 Setup.AutoscaleFixed.ymin(2) = fread(fid,1,'float64');
 Setup.AutoscaleFixed.ymax(1) = fread(fid,1,'float64');
 Setup.AutoscaleFixed.ymax(2) = fread(fid,1,'float64');
 Setup.AutoscaleFixed.ydiv(1) = fread(fid,1,'int32');
 Setup.AutoscaleFixed.ydiv(2) = fread(fid,1,'int32');
 Setup.AutoscaleMethod        = fread(fid,1,'int32');
 
 Setup.FftSize = 16384;
 
 Cutdata.Diameter             = fread(fid,1,'float32');
 Cutdata.Teeth                = fread(fid,1,'int32');
 Cutdata.Chipload             = fread(fid,1,'float32');
 Cutdata.Immersion            = fread(fid,1,'float32');
 Cutdata.Millmode             = fread(fid,1,'int32');
 Cutdata.Milldir              = fread(fid,1,'float32');

 Stability.DataX              = fread(fid,1,'int32');
 Stability.DataY              = fread(fid,1,'int32');
 Stability.FileX              = fread(fid,maxpath,'char');
 Stability.FileY              = fread(fid,maxpath,'char');
 Stability.FreqRange          = fread(fid,2,'float64');
 Stability.Speed              = fread(fid,1,'float32');
 Stability.Power              = fread(fid,1,'float32');
 Stability.Torque             = fread(fid,1,'float32');
 Stability.PowerEn            = fread(fid,1,'int32');
 Stability.TorqueEn           = fread(fid,1,'int32'); 
 Stability.CutStiff           = fread(fid,1,'float32');
 Stability.MatSelect          = fread(fid,1,'int32');
 Stability.MatCount           = fread(fid,1,'int32');
 for i=1:25
  Stability.Title             = fread(fid,strsize,'char');
  Stability.CutStiff          = fread(fid,1,'float32');
 end

 PlotParams.ReIm.Xmin   = fread(fid,1,'float64');
 PlotParams.ReIm.Xmax   = fread(fid,1,'float64');
 PlotParams.ReIm.Xdiv   = fread(fid,1,'int32');
 PlotParams.ReIm.Ymin(1)= fread(fid,1,'float64');
 PlotParams.ReIm.Ymin(2)= fread(fid,1,'float64');
 PlotParams.ReIm.Ymax(1)= fread(fid,1,'float64');
 PlotParams.ReIm.Ymax(2)= fread(fid,1,'float64');
 PlotParams.ReIm.Ydiv(1)= fread(fid,1,'int32');
 PlotParams.ReIm.Ydiv(2)= fread(fid,1,'int32');

 Time.Year        = fread(fid,1,'int16');
 Time.Month       = fread(fid,1,'int16');
 Time.Dayofweek   = fread(fid,1,'int16');
 Time.Day         = fread(fid,1,'int16');
 Time.Hour        = fread(fid,1,'int16');
 Time.minute      = fread(fid,1,'int16');
 Time.Second      = fread(fid,1,'int16');
 Time.Millisecond = fread(fid,1,'int16');

 df = fread(fid,1,'float64');
 G = fread(fid,Setup.FftSize+2,'float32');
 Gamma = fread(fid, Setup.FftSize/2+1, 'float32');
 
elseif (strncmp(Setup.Header, 'TXF VER 2.0', 11))
 TxfVerNo = 2;
 bandwidth=[10e3,5e3,2.5e3,1e3];
 
 Setup.Description = char([fread(fid,descrsize,'char')]');
 Setup.Tool        = char([fread(fid,descrsize,'char')]');
 Setup.Machine     = char([fread(fid,descrsize,'char')]');
 Setup.Cal         = fread(fid,2,'float32');
 Setup.Unit(1,:)   = char([fread(fid,strsize,'char')]');
 Setup.Unit(2,:)   = char([fread(fid,strsize,'char')]);
 index             = fread(fid,2,'int32');
 Setup.Gain        = gain(index+1);
 clear index;
 Setup.Trigger     = fread(fid,1,'float32');
 Setup.Window      = fread(fid,2,'int32');
 Setup.Over        = fread(fid,2,'int32');
 Setup.Under       = fread(fid,2,'int32');
 index             = fread(fid,1,'int32');
 Setup.Bandwidth   = bandwidth(index+1);
 clear index;
 Setup.Average     = fread(fid,1,'int32');
 Setup.Antialias   = fread(fid,1,'int32');
 Setup.Integration = fread(fid,1,'int32');
 Setup.AutoscaleLims.min      = fread(fid,1,'float64');
 Setup.AutoscaleLims.max      = fread(fid,1,'float64');
 Setup.AutoscaleFixed.xmin    = fread(fid,1,'float64');
 Setup.AutoscaleFixed.xmax    = fread(fid,1,'float64');
 Setup.AutoscaleFixed.xdiv    = fread(fid,1,'int32');
 Setup.AutoscaleFixed.ymin(1) = fread(fid,1,'float64');
 Setup.AutoscaleFixed.ymin(2) = fread(fid,1,'float64');
 Setup.AutoscaleFixed.ymax(1) = fread(fid,1,'float64');
 Setup.AutoscaleFixed.ymax(2) = fread(fid,1,'float64');
 Setup.AutoscaleFixed.ydiv(1) = fread(fid,1,'int32');
 Setup.AutoscaleFixed.ydiv(2) = fread(fid,1,'int32');
 Setup.AutoscaleMethod        = fread(fid,1,'int32');
 
 Setup.FftSize = 16384;
 
 Stability.Speed              = fread(fid,1,'float32');
 Stability.Power              = fread(fid,1,'float32');
 Stability.Torque             = fread(fid,1,'float32');
 Stability.PowerEn            = fread(fid,1,'int32');
 Stability.TorqueEn           = fread(fid,1,'int32'); 
 Stability.Diameter           = fread(fid,1,'float32');
 Stability.Teeth              = fread(fid,1,'int32');
 Stability.Immersion          = fread(fid,1,'float32');
 Stability.Chipload           = fread(fid,1,'float32');
 Stability.CutStiff           = fread(fid,1,'float32');
 Stability.MatSelect          = fread(fid,1,'int32');
 Stability.MatCount           = fread(fid,1,'int32');
 for i=1:25
  Stability.Title             = fread(fid,strsize,'char');
  Stability.CutStiff          = fread(fid,1,'float32');
 end

 PlotParams.ReIm.Xmin   = fread(fid,1,'float64');
 PlotParams.ReIm.Xmax   = fread(fid,1,'float64');
 PlotParams.ReIm.Xdiv   = fread(fid,1,'int32');
 PlotParams.ReIm.Ymin(1)= fread(fid,1,'float64');
 PlotParams.ReIm.Ymin(2)= fread(fid,1,'float64');
 PlotParams.ReIm.Ymax(1)= fread(fid,1,'float64');
 PlotParams.ReIm.Ymax(2)= fread(fid,1,'float64');
 PlotParams.ReIm.Ydiv(1)= fread(fid,1,'int32');
 PlotParams.ReIm.Ydiv(2)= fread(fid,1,'int32');

 Time.Year        = fread(fid,1,'int16');
 Time.Month       = fread(fid,1,'int16');
 Time.Dayofweek   = fread(fid,1,'int16');
 Time.Day         = fread(fid,1,'int16');
 Time.Hour        = fread(fid,1,'int16');
 Time.minute      = fread(fid,1,'int16');
 Time.Second      = fread(fid,1,'int16');
 Time.Millisecond = fread(fid,1,'int16');

 df = fread(fid,1,'float64');
 G = fread(fid,Setup.FftSize+2,'float32');
 Gamma = fread(fid, Setup.FftSize/2+1, 'float32');
 
elseif (strncmp(Setup.Header, 'TXF VER 1.0', 11))
 TxfVerNo = 1;
 bandwidth=[10e3,5e3,2.5e3,1e3];
 
 Setup.Description = char([fread(fid,descrsize,'char')]');
 Setup.Cal         = fread(fid,2,'float32');
 Setup.Unit(1,:)   = char([fread(fid,strsize,'char')]');
 Setup.Unit(2,:)   = char([fread(fid,strsize,'char')]);
 index             = fread(fid,2,'int32');
 Setup.Gain        = gain(index+1);
 clear index;
 Setup.Trigger     = fread(fid,1,'float32');
 Setup.Window      = fread(fid,2,'int32');
 Setup.Over        = fread(fid,2,'int32');
 Setup.Under       = fread(fid,2,'int32');
 index             = fread(fid,1,'int32');
 Setup.Bandwidth   = bandwidth(index+1);
 clear index;
 Setup.Average     = fread(fid,1,'int32');
 Setup.Antialias   = fread(fid,1,'int32');
 Setup.Integration = fread(fid,1,'int32');

 Setup.FftSize = 16384;
 
 PlotParams.ReIm.Xmin   = fread(fid,1,'float64');
 PlotParams.ReIm.Xmax   = fread(fid,1,'float64');
 PlotParams.ReIm.Xdiv   = fread(fid,1,'int32');
 PlotParams.ReIm.Ymin(1)= fread(fid,1,'float64');
 PlotParams.ReIm.Ymin(2)= fread(fid,1,'float64');
 PlotParams.ReIm.Ymax(1)= fread(fid,1,'float64');
 PlotParams.ReIm.Ymax(2)= fread(fid,1,'float64');
 PlotParams.ReIm.Ydiv(1)= fread(fid,1,'int32');
 PlotParams.ReIm.Ydiv(2)= fread(fid,1,'int32');
 
 df = fread(fid,1,'float64');
 G = fread(fid,Setup.FftSize+2,'float32');
 Gamma = NaN;
end

[m,n]=size(G);
FRSize=m/2;
f = [0:FRSize-1]'*df;
G = [G(1:2:2*FRSize-1,:) + j*G(2:2:2*FRSize,:)];

% scale frequency response plot
   
[Nsamp, Nvect]=size(G);
if (TxfVerNo ~= 1.0)
 warning off
 for i=1:Nvect
  iMeasLoc = floor((i+1)/2);
  G(:,i) = (Setup.Cal(iMeasLoc,2)/Setup.Cal(iMeasLoc,1))*G(:,i)./((j*2*pi*f).^Setup.Integration(iMeasLoc));
 end
 warning on
end

fclose(fid);


% if there are no return parameters, plot the data on screen with the same limits
% used when the measurement was saved
if nargout == 0

 for i=1:Nvect
  hfig=figure(i);
  
  % plot real part of frequency response
  hax(1)=subplot(2,1,1);
  plot(f,real(G(:,i)))
  axis([PlotParams.ReIm(i).Xmin,PlotParams.ReIm(i).Xmax,PlotParams.ReIm(i).Ymin(1),PlotParams.ReIm(i).Ymax(1)])
  grid

  delta = (PlotParams.ReIm(i).Xmax-PlotParams.ReIm(i).Xmin)/PlotParams.ReIm(i).Xdiv;
  set(hax(1),'XTick',[[PlotParams.ReIm(i).Xmin:delta:PlotParams.ReIm(i).Xmax-delta/2],PlotParams.ReIm(i).Xmax])
  delta = (PlotParams.ReIm(i).Ymax(1)-PlotParams.ReIm(i).Ymin(1))/PlotParams.ReIm(i).Ydiv(1);
  set(hax(1),'YTick',[[PlotParams.ReIm(i).Ymin(1):delta:PlotParams.ReIm(i).Ymax(1)-delta/2],PlotParams.ReIm(i).Ymax(1)])

  hxl(1)=xlabel('Frequency, Hz');
  hyl(1)=ylabel('Real, m/N');
  if (TxfVerNo < 5)
   htl(1)=title(Setup.Description);
  else
   htl(1)=title(MeasType(i));
  end

  % plot imaginary part of frequency response
  hax(2)=subplot(2,1,2);
  plot(f,imag(G(:,i)))
  axis([PlotParams.ReIm(i).Xmin,PlotParams.ReIm(i).Xmax,PlotParams.ReIm(i).Ymin(2),PlotParams.ReIm(i).Ymax(2)])
  grid

  delta = (PlotParams.ReIm(i).Xmax-PlotParams.ReIm(i).Xmin)/PlotParams.ReIm(i).Xdiv;
  set(hax(2),'XTick',[[PlotParams.ReIm(i).Xmin:delta:PlotParams.ReIm(i).Xmax-delta/2],PlotParams.ReIm(i).Xmax])
  delta = (PlotParams.ReIm(i).Ymax(2)-PlotParams.ReIm(i).Ymin(2))/PlotParams.ReIm(i).Ydiv(2);
  set(hax(2),'YTick',[[PlotParams.ReIm(i).Ymin(2):delta:PlotParams.ReIm(i).Ymax(2)-delta/2],PlotParams.ReIm(i).Ymax(2)])

  hxl(2)=xlabel('Frequency, Hz');
  hyl(2)=ylabel('Imaginary, m/N');

  % set font size
  set([hxl,hyl,htl,hax],'FontSize',12);
 end

 % clear return parameters to prevent echoing to the terminal
 clear f G Gamma Setup PlotParams.ReIm
end